@@ -5,6 +5,7 @@ import errno
 import logging
 import os
 import shutil
+import tempfile
 import traceback
 from collections import OrderedDict
 from typing import IO, Any, Callable, Dict, List, MutableMapping, Optional, Union, Iterable
@@ -33,6 +34,14 @@ def get_cache_dir(cache_dir: Optional[str] = None) -> str:
         cache_dir = os.path.expanduser(
             os.getenv("FVCORE_CACHE", "~/.torch/iopath_cache")
         )
+    try:
+        g_pathmgr.mkdirs(cache_dir)
+        assert os.access(cache_dir, os.W_OK)
+    except (OSError, AssertionError):
+        tmp_dir = os.path.join(tempfile.gettempdir(), "iopath_cache")
+        logger = logging.getLogger(__name__)
+        logger.warning(f"{cache_dir} is not accessible! Using {tmp_dir} instead!")
+        cache_dir = tmp_dir
     return cache_dir
 
 
@@ -227,6 +236,21 @@ class PathHandler:
         """
         raise NotImplementedError()
 
+    def _mv(
+        self, src_path: str, dst_path: str, **kwargs: Any
+    ) -> bool:
+        """
+        Moves (renames) a source path to a destination path.
+
+        Args:
+            src_path (str): A URI supported by this PathHandler
+            dst_path (str): A URI supported by this PathHandler
+
+        Returns:
+            status (bool): True on success
+        """
+        raise NotImplementedError()
+
     def _exists(self, path: str, **kwargs: Any) -> bool:
         """
         Checks if there is a resource at the given URI.
@@ -305,6 +329,20 @@ class PathHandler:
         """
         raise NotImplementedError()
 
+    def _set_cwd(self, path: Union[str, None], **kwargs: Any) -> bool:
+        """
+        Set the current working directory. PathHandler classes prepend the cwd
+        to all URI paths that are handled.
+
+        Args:
+            path (str) or None: A URI supported by this PathHandler. Must be a valid
+                absolute Unix path or None to set the cwd to None.
+
+        Returns:
+            bool: true if cwd was set without errors
+        """
+        raise NotImplementedError()
+
 
 class NativePathHandler(PathHandler):
     """
@@ -312,6 +350,8 @@ class NativePathHandler(PathHandler):
     handler uses `open()` and `os.*` calls on the given path.
     """
 
+    _cwd = None
+
     def _get_local_path(self, path: str, **kwargs: Any) -> str:
         self._check_kwargs(kwargs)
         return os.fspath(path)
@@ -320,6 +360,8 @@ class NativePathHandler(PathHandler):
         self, local_path: str, dst_path: str, overwrite: bool = False, **kwargs: Any
     ) -> None:
         self._check_kwargs(kwargs)
+        local_path = self._get_path_with_cwd(local_path)
+        dst_path = self._get_path_with_cwd(dst_path)
         assert self._copy(
             src_path=local_path, dst_path=dst_path, overwrite=overwrite, **kwargs
         )
@@ -377,7 +419,7 @@ class NativePathHandler(PathHandler):
         """
         self._check_kwargs(kwargs)
         return open(  # type: ignore
-            path,
+            self._get_path_with_cwd(path),
             mode,
             buffering=buffering,
             encoding=encoding,
@@ -402,7 +444,8 @@ class NativePathHandler(PathHandler):
             status (bool): True on success
         """
         self._check_kwargs(kwargs)
-
+        src_path = self._get_path_with_cwd(src_path)
+        dst_path = self._get_path_with_cwd(dst_path)
         if os.path.exists(dst_path) and not overwrite:
             logger = logging.getLogger(__name__)
             logger.error("Destination file {} already exists.".format(dst_path))
@@ -416,6 +459,35 @@ class NativePathHandler(PathHandler):
             logger.error("Error in file copy - {}".format(str(e)))
             return False
 
+    def _mv(
+        self, src_path: str, dst_path: str, **kwargs: Any
+    ) -> bool:
+        """
+        Moves (renames) a source path to a destination path.
+
+        Args:
+            src_path (str): A URI supported by this PathHandler
+            dst_path (str): A URI supported by this PathHandler
+
+        Returns:
+            status (bool): True on success
+        """
+        self._check_kwargs(kwargs)
+        src_path = self._get_path_with_cwd(src_path)
+        dst_path = self._get_path_with_cwd(dst_path)
+        if os.path.exists(dst_path):
+            logger = logging.getLogger(__name__)
+            logger.error("Destination file {} already exists.".format(dst_path))
+            return False
+
+        try:
+            shutil.move(src_path, dst_path)
+            return True
+        except Exception as e:
+            logger = logging.getLogger(__name__)
+            logger.error("Error in move operation - {}".format(str(e)))
+            return False
+
     def _symlink(self, src_path: str, dst_path: str, **kwargs: Any) -> bool:
         """
         Creates a symlink to the src_path at the dst_path
@@ -428,6 +500,8 @@ class NativePathHandler(PathHandler):
             status (bool): True on success
         """
         self._check_kwargs(kwargs)
+        src_path = self._get_path_with_cwd(src_path)
+        dst_path = self._get_path_with_cwd(dst_path)
         logger = logging.getLogger(__name__)
         if not os.path.exists(src_path):
             logger.error("Source path {} does not exist".format(src_path))
@@ -444,19 +518,19 @@ class NativePathHandler(PathHandler):
 
     def _exists(self, path: str, **kwargs: Any) -> bool:
         self._check_kwargs(kwargs)
-        return os.path.exists(path)
+        return os.path.exists(self._get_path_with_cwd(path))
 
     def _isfile(self, path: str, **kwargs: Any) -> bool:
         self._check_kwargs(kwargs)
-        return os.path.isfile(path)
+        return os.path.isfile(self._get_path_with_cwd(path))
 
     def _isdir(self, path: str, **kwargs: Any) -> bool:
         self._check_kwargs(kwargs)
-        return os.path.isdir(path)
+        return os.path.isdir(self._get_path_with_cwd(path))
 
     def _ls(self, path: str, **kwargs: Any) -> List[str]:
         self._check_kwargs(kwargs)
-        return os.listdir(path)
+        return os.listdir(self._get_path_with_cwd(path))
 
     def _mkdirs(self, path: str, **kwargs: Any) -> None:
         self._check_kwargs(kwargs)
@@ -471,6 +545,25 @@ class NativePathHandler(PathHandler):
         self._check_kwargs(kwargs)
         os.remove(path)
 
+    def _set_cwd(self, path: Union[str, None], **kwargs: Any) -> bool:
+        self._check_kwargs(kwargs)
+        # Remove cwd path if None
+        if path is None:
+            self._cwd = None
+            return True
+
+        # Make sure path is a valid Unix path
+        if not os.path.exists(path):
+            raise ValueError(f"{path} is not a valid Unix path")
+        # Make sure path is an absolute path
+        if not os.path.isabs(path):
+            raise ValueError(f"{path} is not an absolute path")
+        self._cwd = path
+        return True
+
+    def _get_path_with_cwd(self, path: str) -> str:
+        return path if not self._cwd else os.path.join(self._cwd, path)
+
 
 class HTTPURLHandler(PathHandler):
     """
@@ -532,7 +625,7 @@ class HTTPURLHandler(PathHandler):
         return open(local_path, mode)
 
 
-class OneDrivePathHandler(PathHandler):
+class OneDrivePathHandler(HTTPURLHandler):
     """
     Map OneDrive (short) URLs to direct download links
     """
@@ -561,24 +654,6 @@ class OneDrivePathHandler(PathHandler):
     def _get_supported_prefixes(self) -> List[str]:
         return [self.ONE_DRIVE_PREFIX]
 
-    def _open(
-        self, path: str, mode: str = "r", buffering: int = -1, **kwargs: Any
-    ) -> Union[IO[str], IO[bytes]]:
-        """
-        Open a remote OneDrive path. The resource is first downloaded and cached
-        locally.
-
-        Args:
-            path (str): A OneDrive URI supported by this PathHandler
-            mode (str): Specifies the mode in which the file is opened. It defaults
-                to 'r'.
-            buffering (int): Not used for this PathHandler.
-
-        Returns:
-            file: a file-like object.
-        """
-        return PathManager.open(self._get_local_path(path), mode, **kwargs)
-
     def _get_local_path(self, path: str, **kwargs: Any) -> str:
         """
         This implementation downloads the remote resource and caches it locally.
@@ -589,7 +664,7 @@ class OneDrivePathHandler(PathHandler):
 
         logger.info(f"URL {path} mapped to direct download link {direct_url}")
 
-        return PathManager.get_local_path(os.fspath(direct_url), **kwargs)
+        return super()._get_local_path(os.fspath(direct_url), **kwargs)
 
 
 class PathManager:
@@ -612,6 +687,12 @@ class PathManager:
         A NativePathHandler that works on posix paths. This is used as the fallback.
         """
 
+        self._cwd: Optional[str] = None
+        """
+        Keeps track of the single cwd (if set).
+        NOTE: Only one PathHandler can have a cwd set at a time.
+        """
+
     def __get_path_handler(self, path: Union[str, os.PathLike]) -> PathHandler:
         """
         Finds a PathHandler that supports the given path. Falls back to the native
@@ -695,6 +776,34 @@ class PathManager:
             src_path, dst_path, overwrite, **kwargs
         )
 
+    def mv(
+        self, src_path: str, dst_path: str, **kwargs: Any
+    ) -> bool:
+        """
+        Moves (renames) a source path supported by NativePathHandler to
+        a destination path.
+
+        Args:
+            src_path (str): A URI supported by NativePathHandler
+            dst_path (str): A URI supported by NativePathHandler
+
+        Returns:
+            status (bool): True on success
+        Exception:
+            Asserts if both the src and dest paths are not supported by
+            NativePathHandler.
+        """
+
+        # Moving across handlers is not supported.
+        assert self.__get_path_handler(  # type: ignore
+            src_path
+        ) == self.__get_path_handler(
+            dst_path
+        ), "Src and dest paths must be supported by the same path handler."
+        return self.__get_path_handler(src_path)._mv(
+            src_path, dst_path, **kwargs
+        )
+
     def get_local_path(self, path: str, **kwargs: Any) -> str:
         """
         Get a filepath which is compatible with native Python I/O such as `open`
@@ -829,6 +938,25 @@ class PathManager:
         ) == self.__get_path_handler(dst_path)
         return self.__get_path_handler(src_path)._symlink(src_path, dst_path, **kwargs)
 
+    def set_cwd(self, path: Union[str, None], **kwargs: Any) -> bool:
+        """
+        Set the current working directory. PathHandler classes prepend the cwd
+        to all URI paths that are handled.
+
+        Args:
+            path (str) or None: A URI supported by this PathHandler. Must be a valid
+                absolute Unix path or None to set the cwd to None.
+
+        Returns:
+            bool: true if cwd was set without errors
+        """
+        if path is None and self._cwd is None:
+            return True
+        if self.__get_path_handler(path or self._cwd)._set_cwd(path, **kwargs):  # type: ignore
+            self._cwd = path
+            return True
+        return False
+
     def register_handler(
         self, handler: PathHandler, allow_override: bool = False
     ) -> None:
